PROGRAM rationality

! Two players, no covariates, level-1 rationality
!
! This file generates counterexamples for the level-1 rationality entry
! game. It compares the identification regions for d1,d2 using the ALT
! inequalities only, and using the full set of inequalities.
! Input file: normcdf.txt
! output file: rationality.txt
!
! Arie Beresteanu, Ilya Molchanov and Francesca Molinari (Econometrica, 2011): "Sharp Identification Regions in Models with Convex Moment Predictions,"


REAL (KIND=8), DIMENSION(4) :: P
REAL (KIND=8), DIMENSION(2) :: b00,b01,b10,b11
REAL (KIND=8), PARAMETER :: d1star  =   -.6D0
REAL (KIND=8), PARAMETER :: d2star  =   -.7D0
REAL (KIND=8), PARAMETER :: ZERO=0.0D0, ONE=1.0D0, HALF=0.5D0, QUARTER=0.25D0
INTEGER(Kind=2), PARAMETER :: N_grid=501
REAL(KIND=8), DIMENSION(N_grid) :: normcdf,d
REAL (KIND=8) :: P00,P10,P01,P11,d1,d2,n1,n2,norm
INTEGER(KIND=4) :: za,zt,zu,I,J,ios
LOGICAL(KIND=4), DIMENSION(6) :: e
LOGICAL(KIND=4) :: ABJ,CT,BMM

!!!!!!!!!!! readin the normal cdf!!!!!!!!!!!!!!!!!!!!!!!
10 FORMAT (F6.2,F14.10)
OPEN(Unit=9,file='normcdf.txt',IOSTAT=ios)
IF (ios.ne.0) THEN
  PRINT *,'error reading in file'
  STOP
ENDIF

DO I=1,N_grid
    READ(Unit=9,FMT=10) d1,norm
    d(I)=d1
    normcdf(I)=norm
    !print *,d1,norm
END DO
CLOSE(Unit=9)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

P00=0.504794124351433D0
P01=0.199624657336531D0
P10=0.221782295538221D0
P11=0.0737989227738157D0

P = (/P00, P01, P10, P11/)

za      =   ZERO
zt      =   ZERO
zu      =   ZERO
st      =   0.05D0

OPEN(Unit=9,file='rationality.txt',IOSTAT=ios)
IF (ios.ne.0) THEN
  PRINT *,'error reading in file'
  STOP
ENDIF

40 FORMAT (I6, 2(F6.2))

DO I = 1,N_grid
    n1=normcdf(I)
    d1=d(I)
    DO J = 1,N_grid
        ABJ=.FALSE.
        CT =.FALSE.
        BMM=.FALSE.
        n2=normcdf(J)
        d2=d(J)
        b00     =  (/QUARTER,  n1*n2/)
        b01     =  (/HALF*(ONE-n2),  n1*HALF/)
        b10     =  (/(ONE-n1)*HALF,  HALF*n2/)
        b11     =  (/(ONE-n1)*(ONE-n2),  QUARTER/)
        ABJ  =  ((P(1).LE.b00(2)) .AND. (P(2).LE.b01(2)) .AND. (P(3).LE.b10(2)) .AND. (P(4)<=b11(2)))
        IF (ABJ) THEN
            za  =   za+1
            CT  =  (ABJ .AND. (P(1).GE.b00(1)) .AND. (P(2).GE.b01(1)) .AND. (P(3).GE.b10(1)) .AND. (P(4).GE.b11(1)))
            IF (CT) THEN
                zt  =   zt+1
                e   =  (/ (P(1)+P(2).LE.n1), (P(1)+P(3).LE.n2), (P(1)+P(4).LE.ONE-b01(1)-b10(1)), (P(2)+P(3).LE.ONE-b00(1)-b11(1)), (P(2)+P(4).LE.HALF), (P(3)+P(4).LE.HALF) /)
                BMM  =   (CT .AND.ALL(e))
                IF (BMM) THEN
                    zu  =   zu+1
                END IF
            END IF
        END IF
        !writing the results
        IF (BMM) THEN
            WRITE (unit=9,FMT=40) 3,d1, d2
        ELSE
            IF (CT) THEN
                WRITE (unit=9,FMT=40) 2,d1, d2
            ELSE
                IF (ABJ) THEN
                    WRITE (unit=9,FMT=40) 1,d1, d2
                END IF
            END IF
        END IF
    END DO
END DO
PRINT *,za, zt, zu

END PROGRAM rationality




