MODULE mixfunctions

IMPLICIT NONE
SAVE

REAL (kind=8), PARAMETER :: pi=3.141592653589793D0
INTEGER, PARAMETER :: N_epsilon=10000
INTEGER, PARAMETER :: N_tiles=8
REAL, DIMENSION(4) :: P
REAL (kind=8), PARAMETER :: ONE=1.0D0, ZERO=0.0D0, HALF=0.50D0, QUARTER=0.25D0, TWO=2.0D0
REAL (kind=8), PARAMETER :: t=1.00D-07, DLOW=-4.9950D0, DHIGH=-0.0050D0, DSTEP=0.010D0
INTEGER, PARAMETER               :: GLn=32
REAL (kind=8), DIMENSION(GLn)    :: GLx,GLw !Gauss-Legendre grid points and weights
INTEGER, PARAMETER, DIMENSION(2) :: first_seed=(/90971,854099/)
REAL (kind=8), PARAMETER         :: Cp=0.95D0, Cg=0.95D0 
INTEGER, PARAMETER               :: N_iter=1000, Np=40

DATA GLx/.001368073D0,.0071942277D0,.0176188818D0,.0325469453D0, &
     .0518394344D0,.0753161897D0,.1027581033D0,.1339089403D0,.1684778666D0, &
     .2061421214D0,.2465500455D0,.2893243619D0,.3340656989D0,.3803563189D0, &
     .4277640192D0,.4758461672D0,.5241538328D0,.5722359808D0,.6196436811D0, &
     .6659343011D0,.7106756381D0,.7534499545D0,.7938578786D0,.8315221334D0, &
     .8660910597D0,.8972418967D0,.9246838103D0,.9481605656D0,.9674530547D0, &
     .9823811182D0,.9928057723D0,.9986319268D0/
DATA GLw/.003509312D0,.0081372115D0,.0126959771D0,.0171370005D0, &
     .0214179378D0,.0254990642D0,.0293420289D0,.0329111118D0,.0361728923D0, &
     .0390969435D0,.0416559573D0,.0438260415D0,.0455869341D0,.0469221942D0, &
     .0478193546D0,.0482700387D0,.0482700387D0,.0478193546D0,.0469221942D0, &
     .0455869341D0,.0438260415D0,.0416559573D0,.0390969435D0,.0361728923D0, &
     .0329111118D0,.0293420289D0,.0254990642D0,.0214179378D0,.0171370005D0, &
     .0126959771D0,.0081372115D0,.0035093120D0/

CONTAINS


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE is_mix(th1,th2,ABJ,CT,BMM)
! Checks whether (th1,th2) are in the identification region
! th1,th2 - the two scalar parameters
! P - vector of empirical frequencies
REAL (kind=8), DIMENSION(5) :: p_regions
REAL (kind=8), INTENT(IN) :: th1,th2
REAL (kind=8), DIMENSION(4) :: u,LB,UB
LOGICAL, INTENT(OUT) :: ABJ,CT,BMM
REAL (kind=8) :: pa, pb, pc,pd,pe,EX1,EX2,cdf1,cdf2,pdf0,pdf1,pdf2
INTEGER :: j,k


!initial values for the region's probability mass
!first compute regions
CDF1=alnorm(-th1,.FALSE.)
CDF2=alnorm(-th2,.FALSE.)
pdf0=EXP(ZERO)/SQRT(TWO*pi)
pdf1=EXP(-(th1**TWO)/TWO)/SQRT(TWO*pi)
pdf2=EXP(-(th2**TWO)/TWO)/SQRT(TWO*pi)
EX1=(pdf0-pdf1)
EX2=(pdf0-pdf2)

PA=QUARTER
PB=QUARTER+(CDF2-HALF)*(ONE-CDF1)
PC=QUARTER+(ONE-CDF2)*(CDF1-HALF)
PE=(CDF1-HALF)*(CDF2-HALF)
PD=ONE-PA-PB-PC-PE

! check in all N_tiles directions
ABJ=.TRUE.
CT =.TRUE.
BMM=.TRUE.

!first check whether (th1,th2) are in ABJ's or CT's ID region
LB=(/PA,PB,PC,PD/)
UB=LB+(/ (CDF1-HALF+EX1/th1)*(CDF2-HALF+EX2/th2),PE,PE,(EX1/th1)*(EX2/th2) /)

IF (P(1)>UB(1)+t .OR. P(2)>UB(2)+t .OR. P(3)>UB(3)+t .OR. P(4)>UB(4)+t) THEN
    ABJ=.FALSE.
    CT =.FALSE.
    BMM=.FALSE.
ELSE
    IF (P(1)<LB(1)-t .OR. P(2)<LB(2)-t .OR. P(3)<LB(3)-t .OR. P(4)<LB(4)-t) THEN
        CT =.FALSE.
        BMM=.FALSE.
    END IF
END IF

!now check whether it is in BMM's ID region
p_regions=(/ pa, pb, pc, pd, pe/)
IF (BMM) THEN 
    BMM=isminneg2(th1,th2,p_regions) !check if the point is in BMM's ID region only if don't already know that it can be
END IF

END SUBROUTINE is_mix
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!



!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
LOGICAL FUNCTION isminneg2(th1,th2,p_reg)

REAL (kind=8), INTENT(IN) :: th1,th2
REAL (kind=8), INTENT(IN), DIMENSION(5) :: p_reg


REAL (kind=8), DIMENSION(Np,4) :: particles_loc,lbest_loc,velocity
REAL (kind=8), DIMENSION(Np)   :: particles_val,lbest_val
REAL (kind=8), SAVE, DIMENSION(4) :: gbest=ZERO
REAL (kind=8) :: rg,rp,mn
INTEGER :: j,ii(1),jj,k,i,count

!Select a random starting population of Np points
particles_loc(1,1:4)=gbest
CALL RANDOM_NUMBER(particles_loc(2:Np,1:4))
DO j=1,Np
    particles_val(j)=vfGL(particles_loc(j,1:4),th1,th2,p_reg)
END DO

!set initial values for local best
lbest_loc=particles_loc
lbest_val=particles_val

! Select random velocities
CALL RANDOM_NUMBER(velocity)
velocity=TWO*velocity-ONE

!initiate global best
ii=MINLOC(lbest_val)
gbest =lbest_loc(ii(1),:)

count=0
DO
    mn=MINVAL(particles_val) !global best
    isminneg2=(mn.GT.-t)
    IF (.NOT.(isminneg2) .OR. (count.GE.N_iter)) EXIT  !.OR. (mx-mn).LT.t/10.0D0 
    count=count+1
    DO i=1,Np
        !update velocity
        CALL RANDOM_NUMBER(rp)
        CALL RANDOM_NUMBER(rg)
        velocity(i,:)=velocity(i,:)+Cp*rp*(lbest_loc(i,:)-particles_loc(i,:))+Cg*rg*(gbest-particles_loc(i,:))
        particles_loc(i,:)=particles_loc(i,:)+velocity(i,:)
        particles_val(i)=vfGL(particles_loc(i,:),th1,th2,p_reg)
        IF ((particles_val(i).LT.lbest_val(i))) THEN  ! .OR. (rj.LT.(mn/mx))
            lbest_val(i)=particles_val(i)
            lbest_loc(i,:)=particles_loc(i,:)
        END IF
    END DO
    ii=MINLOC(lbest_val)
    gbest =lbest_loc(ii(1),:)
END DO
!PRINT *,th1,th2,count,MINVAL(U(:,5)),isminneg2


END FUNCTION isminneg2
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
REAL (kind=8) FUNCTION vf(u,th1,th2,p_reg)

REAL (kind=8), INTENT(IN), DIMENSION(4) :: u
REAL (kind=8), INTENT(IN)               :: th1,th2
REAL (kind=8), INTENT(IN), DIMENSION(5) :: p_reg

REAL (kind=8) :: val, E
INTEGER :: i

IF (MAX(u(2),u(3)).GE.MAX(u(1),u(4))) THEN
    E=u(1)*p_reg(1)+u(2)*p_reg(2)+u(3)*p_reg(3)+u(4)*p_reg(4)+MAX(u(2),u(3))*p_reg(5)
ELSE
    val=ZERO
    CALL glint(th1,th2,u,val)
    E=u(1)*p_reg(1)+u(2)*p_reg(2)+u(3)*p_reg(3)+u(4)*p_reg(4)+val
END IF
vf=E-DOT_PRODUCT(P,u);
END FUNCTION vf
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE glint(th1,th2,u,val)
! This procedure integrates the function MAX(u(2),u(3),DOT_PRODUCT(u,delta)) over
! the range 0<x1<-th1 and 0<x2<-th2 where delta is a the following function of (x1,x2):
! delta=( (1+x2/th2)*(1+x1/th1) , (-x2/th2)*(1+x1/th1) , (1+x2/th2)*(-x1/th1) , (-x2/th2)*(-x1/th1) ).
! Note that th1 and th2 are both negative numbers. The procedure uses the Gauss-Legendre with 32
! abscissas in each dimension. These abscissas and weights are pre determined and taken from an online
! source.

REAL (kind=8), INTENT(IN)               :: th1,th2
REAL (kind=8), INTENT(IN), DIMENSION(4) :: u
REAL (kind=8), INTENT(OUT)              :: val

INTEGER       :: i,j
REAL (kind=8) :: x1,x2,f,delt(4)

val=ZERO

DO i=1,GLn
!	x1=(-th1)*GLx(i)
	DO j=1,GLn
!		x2=(-th2)*GLx(j)
!		delt=(/ (1+x2/th2)*(1+x1/th1) , (-x2/th2)*(1+x1/th1) , (1+x2/th2)*(-x1/th1) , (-x2/th2)*(-x1/th1) /)
        delt=(/ (1-GLx(j))*(1-GLx(i)) , GLx(j)*(1-GLx(i)) , (1-GLx(j))*GLx(i) , GLx(j)*GLx(i) /)
!		f= Max(u(2),u(3),DOT_PRODUCT(u,delt))*EXP(-x1**TWO/TWO)*EXP(-x2**TWO/TWO)
        f=MAX(u(2),u(3),DOT_PRODUCT(u,delt))*EXP(-(GLx(i)*th1)**TWO/TWO-(GLX(j)*th2)**TWO/TWO)
        val=val+GLw(i)*GLw(j)*f
     END DO
END DO
val=val*(-th1)*(-th2)/(TWO*pi)

END SUBROUTINE glint
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE inssor(U)
!  Sorts U into increasing order (Insertion sort) by the fifth column
      REAL (kind=8), DIMENSION (5,5), Intent (InOut) :: U
      REAL (kind=8), DIMENSION (5) :: XWRK, XMIN
      INTEGER :: ICRS, IDCR
!
! We first bring the minimum to the first location in the array.
! That way, we will have a "guard", and when looking for the
! right place to insert a value, no loop test is necessary.
!
      IF (U(1,5) < U(5,5)) THEN
          XMIN = U(1,1:5)
      ELSE
          XMIN = U(5,1:5)
          U(5,1:5) = U(1,1:5)
      END IF
      DO IDCR = 4, 2, -1
         XWRK = U(IDCR,1:5)
         IF (XWRK(5) < XMIN(5)) THEN
            U(IDCR,1:5) = XMIN
            XMIN = XWRK
         END IF
      END DO
      U(1,1:5) = XMIN
!
! The first value is now the minimum
! Loop over the array, and when a value is smaller than
! the previous one, loop down to insert it at its right place.
!
      DO ICRS = 3, 5
         XWRK = U(ICRS,1:5)
         IDCR = ICRS - 1
         IF (XWRK(5) < U(IDCR,5)) THEN
            U(ICRS,1:5) = U(IDCR,1:5)
            IDCR = IDCR - 1
            DO
               IF (XWRK(5) >= U(IDCR,5)) EXIT
               U(IDCR+1,1:5) = U(IDCR,1:5)
               IDCR = IDCR - 1
            END DO
            U(IDCR+1,1:5) = XWRK
         END IF
      END DO
!
      RETURN
!
END SUBROUTINE inssor
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
FUNCTION alnorm ( x, upper )
!*****************************************************************************
!
!! ALNORM computes the cumulative density of the standard normal distribution.
!
!  Modified:
!
!    13 January 2008
!
!  Author:
!
!    David Hill
!    FORTRAN90 version by John Burkardt
!
!  Reference:
!
!    David Hill,
!    Algorithm AS 66:
!    The Normal Integral,
!    Applied Statistics,
!    Volume 22, Number 3, 1973, pages 424-427.
!
!  Parameters:
!
!    Input, real ( kind = 8 ) X, is one endpoint of the semi-infinite interval
!    over which the integration takes place.
!
!    Input, logical UPPER, determines whether the upper or lower
!    interval is to be integrated:
!    .TRUE.  => integrate from X to + Infinity;
!    .FALSE. => integrate from - Infinity to X.
!
!    Output, real ( kind = 8 ) ALNORM, the integral of the standard normal
!    distribution over the desired interval.
!
  IMPLICIT NONE

  REAL (kind=8), PARAMETER :: a1 = 5.75885480458D+00
  REAL (kind=8), PARAMETER :: a2 = 2.62433121679D+00
  REAL (kind=8), PARAMETER :: a3 = 5.92885724438D+00
  REAL (kind=8) alnorm
  REAL (kind=8), PARAMETER :: b1 = -29.8213557807D+00
  REAL (kind=8), PARAMETER :: b2 = 48.6959930692D+00
  REAL (kind=8), PARAMETER :: c1 = -0.000000038052D+00
  REAL (kind=8), PARAMETER :: c2 = 0.000398064794D+00
  REAL (kind=8), PARAMETER :: c3 = -0.151679116635D+00
  REAL (kind=8), PARAMETER :: c4 = 4.8385912808D+00
  REAL (kind=8), PARAMETER :: c5 = 0.742380924027D+00
  REAL (kind=8), PARAMETER :: c6 = 3.99019417011D+00
  REAL (kind=8), PARAMETER :: con = 1.28D+00
  REAL (kind=8), PARAMETER :: d1 = 1.00000615302D+00
  REAL (kind=8), PARAMETER :: d2 = 1.98615381364D+00
  REAL (kind=8), PARAMETER :: d3 = 5.29330324926D+00
  REAL (kind=8), PARAMETER :: d4 = -15.1508972451D+00
  REAL (kind=8), PARAMETER :: d5 = 30.789933034D+00
  REAL (kind=8), PARAMETER :: ltone = 7.0D+00
  REAL (kind=8), PARAMETER :: p = 0.398942280444D+00
  REAL (kind=8), PARAMETER :: q = 0.39990348504D+00
  REAL (kind=8), PARAMETER :: r = 0.398942280385D+00
  LOGICAL :: up
  LOGICAL :: upper
  REAL (kind=8), PARAMETER :: utzero = 18.66D+00
  REAL (kind=8) x
  REAL (kind=8) y
  REAL (kind=8) z

  up = upper
  z = x

  IF ( z < 0.0D+00 ) THEN
    up = .not. up
    z = - z
  END IF

  IF ( ltone < z .and. ( ( .not. up ) .or. utzero < z ) ) THEN

    IF ( up ) THEN
      alnorm = 0.0D+00
    ELSE
      alnorm = 1.0D+00
    END IF

    RETURN

  END IF

  y = 0.5D+00 * z * z

  IF ( z <= con ) THEN

    alnorm = 0.5D+00 - z * ( p - q * y &
      / ( y + a1 + b1 &
      / ( y + a2 + b2 & 
      / ( y + a3 ))))

  ELSE

    alnorm = r * exp ( - y ) &
      / ( z + c1 + d1 &
      / ( z + c2 + d2 &
      / ( z + c3 + d3 &
      / ( z + c4 + d4 &
      / ( z + c5 + d5 &
      / ( z + c6 ))))))

  END IF

  IF ( .not. up ) THEN
    alnorm = 1.0D+00 - alnorm
  END IF

  RETURN
END FUNCTION alnorm
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
FUNCTION norminv ( p )

!*****************************************************************************80
!
!! norminv inverts the standard normal CDF.
!
!  Discussion:
!
!    The result is accurate to about 1 part in 10**16.
!
!  Modified:
!
!    27 December 2004
!
!  Author:
!
!    Michael Wichura.
!    FORTRAN90 version by John Burkardt
!
!  Reference:
!
!    Michael Wichura,
!    The Percentage Points of the Normal Distribution,
!    Algorithm AS 241,
!    Applied Statistics,
!    Volume 37, Number 3, pages 477-484, 1988.
!
!  Parameters:
!
!    Input, real ( kind = 8 ) P, the value of the cumulative probability 
!    densitity function.  0 < P < 1.  If P is outside this range,
!    an "infinite" value will be returned.
!
!    Output, real ( kind = 8 ) D_NORMAL_01_CDF_INVERSE, the normal deviate 
!    value with the property that the probability of a standard normal 
!    deviate being less than or equal to the value is P.
!
  IMPLICIT NONE

  REAL    ( kind = 8 ), PARAMETER, DIMENSION ( 8 ) :: a = (/ &
    3.3871328727963666080D+00, &
    1.3314166789178437745D+02, &
    1.9715909503065514427D+03, &
    1.3731693765509461125D+04, &
    4.5921953931549871457D+04, &
    6.7265770927008700853D+04, &
    3.3430575583588128105D+04, &
    2.5090809287301226727D+03 /)
  REAL    ( kind = 8 ), PARAMETER, DIMENSION ( 8 ) :: b = (/ &
    1.0D+00, &
    4.2313330701600911252D+01, &
    6.8718700749205790830D+02, &
    5.3941960214247511077D+03, &
    2.1213794301586595867D+04, &
    3.9307895800092710610D+04, &
    2.8729085735721942674D+04, &
    5.2264952788528545610D+03 /)
  REAL   ( kind = 8 ), PARAMETER, DIMENSION ( 8 ) :: c = (/ &
    1.42343711074968357734D+00, &
    4.63033784615654529590D+00, &
    5.76949722146069140550D+00, &
    3.64784832476320460504D+00, &
    1.27045825245236838258D+00, &
    2.41780725177450611770D-01, &
    2.27238449892691845833D-02, &
    7.74545014278341407640D-04 /)
  REAL    ( kind = 8 ), PARAMETER :: const1 = 0.180625D+00
  REAL    ( kind = 8 ), PARAMETER :: const2 = 1.6D+00
  REAL    ( kind = 8 ), PARAMETER, DIMENSION ( 8 ) :: d = (/ &
    1.0D+00, &
    2.05319162663775882187D+00, &
    1.67638483018380384940D+00, &
    6.89767334985100004550D-01, &
    1.48103976427480074590D-01, &
    1.51986665636164571966D-02, &
    5.47593808499534494600D-04, &
    1.05075007164441684324D-09 /)
  REAL    ( kind = 8 ), PARAMETER, DIMENSION ( 8 ) :: e = (/ &
    6.65790464350110377720D+00, &
    5.46378491116411436990D+00, &
    1.78482653991729133580D+00, &
    2.96560571828504891230D-01, &
    2.65321895265761230930D-02, &
    1.24266094738807843860D-03, &
    2.71155556874348757815D-05, &
    2.01033439929228813265D-07 /)
  REAL    ( kind = 8 ), PARAMETER, DIMENSION ( 8 ) :: f = (/ &
    1.0D+00, &
    5.99832206555887937690D-01, &
    1.36929880922735805310D-01, &
    1.48753612908506148525D-02, &
    7.86869131145613259100D-04, &
    1.84631831751005468180D-05, &
    1.42151175831644588870D-07, &
    2.04426310338993978564D-15 /)
  REAL    ( kind = 8 ) p
  REAL    ( kind = 8 ) q
  REAL    ( kind = 8 ) r
  REAL    ( kind = 8 ) norminv 
  REAL    ( kind = 8 ) r8poly_value
  REAL    ( kind = 8 ), PARAMETER :: split1 = 0.425D+00
  REAL    ( kind = 8 ), PARAMETER :: split2 = 5.0D+00

  IF ( p <= 0.0D+00 ) THEN
    norminv = - HUGE ( p )
    RETURN
  END IF

  IF ( 1.0D+00 <= p ) THEN
    norminv = HUGE ( p )
    RETURN
  END IF

  q = p - 0.5D+00

  IF ( ABS ( q ) <= split1 ) THEN

    r = const1 - q * q
    norminv = q * r8poly_value ( 8, a, r ) &
                                 / r8poly_value ( 8, b, r )

  ELSE

    IF ( q < 0.0D+00 ) THEN
      r = p
    ELSE
      r = 1.0D+00 - p
    END IF

    IF ( r <= 0.0D+00 ) THEN
      norminv = - 1.0D+00
      STOP
    END IF

    r = sqrt ( -log ( r ) )

    IF ( r <= split2 ) THEN

      r = r - const2
      norminv = r8poly_value ( 8, c, r ) &
                               / r8poly_value ( 8, d, r )

    ELSE

      r = r - split2
      norminv = r8poly_value ( 8, e, r ) &
                               / r8poly_value ( 8, f, r )
   
    END IF

    IF ( q < 0.0D+00 ) THEN
      norminv = - norminv
    END IF

  END IF

  RETURN
END FUNCTION norminv

FUNCTION r8poly_value ( n, a, x )

!**************************************************************************
!
!! R8POLY_VALUE evaluates an R8POLY
!
!  Discussion:
!
!    For sanity's sake, the value of N indicates the NUMBER of 
!    coefficients, or more precisely, the ORDER of the polynomial,
!    rather than the DEGREE of the polynomial.  The two quantities
!    dIFfer by 1, but cause a great deal of confusion.
!
!    Given N and A, the form of the polynomial is:
!
!      p(x) = a(1) + a(2) * x + ... + a(n-1) * x^(n-2) + a(n) * x^(n-1)
!
!  ModIFied:
!
!    13 August 2004
!
!  Author:
!
!    John Burkardt
!
!  Parameters:
!
!    Input, INTEGER ( kind = 4 ) N, the order of the polynomial.
!
!    Input, REAL ( kind = 8 ) A(N), the coefficients of the polynomial.
!    A(1) is the constant term.
!
!    Input, REAL ( kind = 8 ) X, the point at which the polynomial is 
!    to be evaluated.
!
!    Output, REAL ( kind = 8 ) R8POLY_VALUE, the value of the polynomial at X.
!
  IMPLICIT NONE

  INTEGER ( kind = 4 ) n

  REAL    ( kind = 8 ) a(n)
  INTEGER ( kind = 4 ) i
  REAL    ( kind = 8 ) r8poly_value
  REAL    ( kind = 8 ) x

  r8poly_value = 0.0D+00
  DO i = n, 1, -1
    r8poly_value = r8poly_value * x + a(i)
  END DO

  RETURN
END FUNCTION r8poly_value
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
REAL(8) FUNCTION vfGL(u,th1,th2,pp)

REAL (8), INTENT(IN)  :: u(4),th1,th2,pp(5)
REAL (8)              :: u1,u2,u3,u4,a,b,c,um,E_singleton,E_GL
REAL (8)              :: k,jj,g,AA,BB,CC,DD,integrand_tot,int_value,temp1,temp2
INTEGER               :: i

!Created: 04/07/10
!Last modified: 04/15/10
!
! This function evaluates E(h(Q(S_{\theta})),u) using quadrature and
! solving one of the two integrals in closed form.  See
! speed_improvements_algorithm.tex for details

u1 = u(1)
u2 = u(2)
u3 = u(3)
u4 = u(4)
a  = u1-u2-u3+u4
b  = u3-u1
c  = u2-u1
um = MAX(u2,u3)

E_singleton = u1*pp(1)+u2*pp(2)+u3*pp(3)+u4*pp(4)

IF (um .GE. MAX(u1,u4)) THEN
    E_GL = um*pp(5)
ELSEIF ((a .EQ. ZERO) .AND. (b .EQ. ZERO) .AND. (c .EQ. ZERO)) THEN
    E_GL = um*pp(5)
ELSEIF ((a .EQ. ZERO) .AND. (b .EQ. ZERO) .AND. (c .NE. ZERO)) THEN
    IF ((c .GT. ZERO) .AND. (um .GT. u1)) THEN
        temp1=ONE
    ELSE
        temp1=ZERO
    ENDIF
    IF ((c .LT. ZERO) .AND. (((um-u1)/c) .LT. ONE)) temp1=temp1+ONE
    IF ((c .LT. ZERO) .AND. (um .LT. u1)) THEN
        temp2=ONE
    ELSE
        temp2=ZERO
    ENDIF
    IF ((c .GT. ZERO) .AND. (((um-u1)/c) .LT. ONE)) temp2=temp2+ONE
    E_GL = um*pp(5)*temp1 + &
        (alnorm(-th1,.FALSE.)-HALF)*(c*(normpdf(ZERO)-normpdf(-th2))/(-th2) + u1*(alnorm(-th2,.FALSE.)-HALF)) * temp2
ELSE
    E_GL = ZERO
    DO i=1,32
        k = (a*GLx(i) + b)
        jj= (c*GLx(i) + u1)
        g = (um-u1-c*GLx(i))/k

        AA = ((ONE/(-th1))*k*(normpdf(MAX(ZERO,-th1*g))-normpdf(-th1))+ &
            jj*(alnorm(-th1,.FALSE.)-alnorm(MAX(ZERO,-th1*g),.FALSE.)))
        BB = ((ONE/(-th1))*k*(normpdf(ZERO)-normpdf(-th1*MIN(g,ONE)))+ &
            jj*(alnorm(-th1*MIN(g,ONE),.FALSE.)-HALF))
        CC = (alnorm(-th1*MIN(g,ONE),.FALSE.)-HALF)
        DD = (alnorm(-th1,.FALSE.)-alnorm(MAX(ZERO,-th1*g),.FALSE.))
        integrand_tot = um*(k .GT. ZERO)*(g .GT. ZERO)*CC + um*(k .LT. ZERO)*(g .LT. ONE)*DD + &
            (k .GT. ZERO)*(g .LT. ONE)*AA + (k .LT. ZERO)*(g .GT. ZERO)*BB
        E_GL = E_GL+integrand_tot*normpdf(GLx(i)*(-th2))*GLw(i)*(-th2)
END DO
ENDIF

vfGL=E_singleton + E_GL-DOT_PRODUCT(P,u)

END FUNCTION vfGL
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
REAL (kind=8) FUNCTION normpdf(x)

REAL (kind=8), INTENT(IN) :: x

normpdf=(ONE/SQRT(2*pi))*EXP(-x**TWO/TWO)

END FUNCTION normpdf
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE random_pick(len,indx)
! this subroutine randomly picks n numbers from (/1,2,...,Np/) without 
! replacement. 
INTEGER, INTENT(IN)  :: len
INTEGER, INTENT(OUT) :: indx(len)
REAL(kind=8)         :: r
INTEGER              :: temp(Np),t,i

temp=(/(i,i=1,Np)/)

DO i=1,len
    !pick a random index
    CALL RANDOM_NUMBER(r)
    t=FLOOR((Np-i+1)*r)+1
    indx(i)=temp(t)
    !move the chosen index to the end of the list
    IF (t.NE.Np-i+1) THEN
        temp(t)=temp(t)+temp(Np-i+1)
        temp(Np-i+1)=temp(t)-temp(Np-i+1)
        temp(t)=temp(t)-temp(Np-i+1)
    END IF
END DO

END SUBROUTINE random_pick
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


END MODULE mixfunctions