PROGRAM mixnash

! Two players, no covariates, Nash Equilibrium
! This program uses the module mixfunctions.f90 
! The program produces the output file points.txt
!
! Arie Beresteanu, Ilya Molchanov and Francesca Molinari (Econometrica, 2011): "Sharp Identification Regions in Models with Convex Moment Predictions,"

USE mixfunctions
IMPLICIT NONE

REAL (kind=8) :: theta1,theta2, PA,PB,PC,PD,PE,P00,P10,P01,P11
REAL (kind=8) :: th1,th2,e1,e2,CDF1,CDF2,EX1,EX2,alpha_m,alpha_01,alpha_10,pdf0,pdf1,pdf2
REAL (kind=8) :: ABJth1min,ABJth1max,ABJth2min,ABJth2max,CTth1min,CTth1max,CTth2min,CTth2max,BMMth1min,BMMth1max,BMMth2min,BMMth2max
REAL (kind=8) :: start,finish,direction
INTEGER :: n_BMMmix,n_CTmix,n_ABJmix,ios,I
LOGICAL :: BMM,ABJ,CT
CHARACTER*10 :: TIME
CHARACTER*8  :: DATE
CHARACTER*15 :: filename


30 FORMAT (2(F20.15))
35 FORMAT (2(F20.15),3(I8))
40 FORMAT (2(I8),2(F20.15))
55 FORMAT (4(F8.5),3(I8),12(F8.5))

OPEN(Unit=6,file='points.txt',IOSTAT=ios)
IF (ios.ne.0) THEN
  PRINT *,'error writting output file'
  STOP
ENDIF


! pick true parameters
DO theta1=-1.80D0,-0.75D0,0.2D0
DO theta2=-1.7D0,-0.650D0,0.2D0
PRINT *, theta1,theta2
!compute several constants

!Compute the probability mass of the various areas in B&R diagram
!
!                                |
!            PC                  |      PD
!                 -theta2+-------+-----------------
!                        |       |
!                        |  PE   |
!    ------------------0---------+
!                        0     -theta1
!           PA           |         
!                        |           PB
!                        |
!
CDF1=alnorm(-theta1,.FALSE.)
CDF2=alnorm(-theta2,.FALSE.)
pdf0=EXP(ZERO)/SQRT(TWO*pi)
pdf1=EXP(-(theta1**TWO)/TWO)/SQRT(TWO*pi)
pdf2=EXP(-(theta2**TWO)/TWO)/SQRT(TWO*pi)
EX1=(pdf0-pdf1)
EX2=(pdf0-pdf2)
!PRINT *,pdf1,pdf2
!PRINT *,EX1,EX2

PA=QUARTER
PB=QUARTER+(CDF2-HALF)*(ONE-CDF1)
PC=QUARTER+(ONE-CDF2)*(CDF1-HALF)
PE=(CDF1-HALF)*(CDF2-HALF)
PD=ONE-PA-PB-PC-PE
!PRINT *,PA,PB,PC,PD,PE

DO alpha_01=ZERO,0.99D0,0.25D0
DO alpha_10=ZERO,ONE-alpha_01,0.25D0
alpha_m =ONE-alpha_01-alpha_10
IF (alpha_m.LT.0.01D0) EXIT
!projections
ABJth1min=DHIGH
ABJth2min=DHIGH
CTth1min=DHIGH
CTth2min=DHIGH
BMMth1min=DHIGH
BMMth2min=DHIGH
ABJth1max=DLOW
ABJth2max=DLOW
CTth1max=DLOW
CTth2max=DLOW
BMMth1max=DLOW
BMMth2max=DLOW




! create a selection
P00=PA+alpha_m*(CDF1-HALF+EX1/theta1)*(CDF2-HALF+EX2/theta2)
P01=PC+alpha_01*PE+alpha_m*(-EX1/theta1)*(CDF2-HALF+EX2/theta2)
P10=PB+alpha_10*PE+alpha_m*(CDF1-HALF+EX1/theta1)*(-EX2/theta2)
P11=ONE-P00-P10-P01
P=(/P00, P10, P01, P11/)
!PRINT *,P

PRINT *,'----------------------------------------------------------'
!PRINT *,'Starting to explore the identification region'
PRINT *,'True parameter:'
PRINT *,theta1,theta2,alpha_01,alpha_10
PRINT *,'  '
PRINT *,'Start time:'
CALL DATE_AND_TIME(DATE,TIME)
PRINT *,TIME
PRINT *,'----------------------------------------------------------'

! provide a parameter candidates to check
n_BMMmix=0 !set the initial length of BMM_mix (i.e. how may parameters were found so far)
n_CTmix =0
n_ABJmix=0

start=DLOW
finish=DHIGH
direction=DSTEP

DO th1=DLOW,DHIGH,DSTEP
    DO th2=start,finish,direction !DLOW,DHIGH,DSTEP
        CALL is_mix(th1,th2,ABJ,CT,BMM)
        IF (ABJ.EQ..TRUE.) THEN
            n_ABJmix=n_ABJmix+1
            ABJth1min=MIN(ABJth1min,th1)
            ABJth2min=MIN(ABJth2min,th2)
            ABJth1max=MAX(ABJth1max,th1)
            ABJth2max=MAX(ABJth2max,th2)
            !WRITE(Unit=6,FMT=40) 1,n_ABJmix,th1,th2
        END IF
        IF (CT.EQ..TRUE.) THEN
            n_CTmix=n_CTmix+1
            CTth1min=MIN(CTth1min,th1)
            CTth2min=MIN(CTth2min,th2)
            CTth1max=MAX(CTth1max,th1)
            CTth2max=MAX(CTth2max,th2)
            !WRITE(Unit=6,FMT=40) 2,n_CTmix,th1,th2
        END IF
        IF (BMM.EQ..TRUE.) THEN
            n_BMMmix=n_BMMmix+1
            BMMth1min=MIN(BMMth1min,th1)
            BMMth2min=MIN(BMMth2min,th2)
            BMMth1max=MAX(BMMth1max,th1)
            BMMth2max=MAX(BMMth2max,th2)
            !WRITE(Unit=6,FMT=40) 3,n_BMMmix,th1,th2
        END IF
    END DO
    direction=-direction
    start=start+finish
    finish=start-finish
    start=start-finish
    !PRINT *,th1, n_BMMmix, n_CTmix,n_ABJmix
END DO

PRINT *,' '
PRINT *,'done simulations'
PRINT *,' '
PRINT *,'points were found under BMM,CT and ABJ mixed estimator: '
PRINT *,n_BMMmix,n_CTmix,n_ABJmix
WRITE(Unit=6,FMT=55) theta1,theta2,alpha_01,alpha_10,n_ABJmix,n_CTmix,n_BMMmix,ABJth1min,ABJth1max,ABJth2min,ABJth2max,CTth1min,CTth1max,CTth2min,CTth2max,BMMth1min,BMMth1max,BMMth2min,BMMth2max
PRINT *,'--------------------------------------'
PRINT *,'End time:'
CALL DATE_AND_TIME(DATE,TIME)
PRINT *,TIME
PRINT *,'--------------------------------------'

END DO !alpha_01
END DO !alpha_10

END DO !theta2
END DO !theta1

END PROGRAM mixnash