PROGRAM mixnash

! Two players, with covariates, Nash Equilibrium
!
! This is the main program that computes the identification regions under
! the ABJ, CT and BMM methods. The payoff functions include competitive
! effects (thetas) and covariance dependent effect (x'beta). An intercept 
! is included.
! This program uses the module functionsDGP3.f90
! The program produces the output file quickDGP3.txt
!
! Arie Beresteanu, Ilya Molchanov and Francesca Molinari (Econometrica, 2011): "Sharp Identification Regions in Models with Convex Moment Predictions,"

USE mixfunctions
IMPLICIT NONE

!define variables
REAL (kind=8), DIMENSION(par_dim) :: y,ytemp,Center
REAL (kind=8), DIMENSION(par_dim) :: ABJmin,ABJmax,CTmin,CTmax,BMMmin,BMMmax
REAL (kind=8), DIMENSION(cov_num1):: beta1
REAL (kind=8), DIMENSION(cov_num2):: beta2
REAL (kind=8)                     :: theta1,theta2, PA,PB,PC,PD,PE,P00,P10,P01,P11,shift1,shift2,Ptemp(4)
REAL (kind=8)                     :: th1,th2,e1,e2,CDF1,CDF2,CDF3,CDF4,alpha_01,alpha_10,pdf0,pdf1,pdf2,h 
!! CLS parameters
REAL (kind=8)                     :: gamma(par_dim),lambda,currentdist,maxdist
REAL (kind=8), PARAMETER          :: acc=4.0D0
INTEGER, PARAMETER                :: N_CLS=500
!!
INTEGER                           :: n_BMMmix,n_CTmix,n_ABJmix,ios,j,i,k,count,m,m1,m2,m3,METHOD
LOGICAL                           :: BMM,ABJ,CT,BMMtemp,CTtemp,ABJtemp
CHARACTER*10                      :: TIME
CHARACTER*8                       :: DATE
CHARACTER*15                      :: filename

CALL RANDOM_SEED(put=first_seed) !initiate the seed for the random draws

!!!!!!!!!!!!!!!!!!! output file !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
40 FORMAT (1(I8),8(F20.15))
55 FORMAT (4(F8.5),3(I8),24(F8.5))

OPEN(Unit=6,file='quick.txt',IOSTAT=ios)
IF (ios.ne.0) THEN
  PRINT *,'error writting output file'
  STOP
ENDIF
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!projections: (not used here)
ABJmin=DHIGH
CTmin=ABJmin
BMMmin=ABJmin
ABJmax=DLOW
CTmax=ABJmax
BMMmax=ABJmax
!counters:
n_BMMmix=0 
n_CTmix =0
n_ABJmix=0
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!!!!!!!!!!!!!!!!!!! parameters !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!pick payoff parameters
theta1=-1.20D0
theta2=-1.50D0
beta1=(/HALF, ONE/3.0D0, ZERO/)
beta2=(/-ONE/3.0D0, -HALF, ZERO/)
!pick selection parameters
alpha_01=0.25D0
alpha_10=0.00D0
!alpha_m =ONE-alpha_01-alpha_10

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!! compute P(Y|X) !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
DO m1=1,cov_val1
    DO m2=1,cov_val2
            shift1=DOT_PRODUCT(X1(m1,:),beta1)
            shift2=DOT_PRODUCT(X2(m2,:),beta2)
            CALL selection(theta1,theta2,shift1,shift2,alpha_01,alpha_10,Ptemp) !Ptemp is the output here
            P(m1,m2,:)=Ptemp
    END DO
END DO
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
PRINT *,'# of parameters: ',par_dim

!!!! CLS parameters
lambda=ONE
gamma=(/0.1123D0, 0.2927D0, 0.3527D0, 0.4871D0, 0.5471D0, 0.6619D0, 0.7229D0, 0.8053D0/)


PRINT *,'----------------------------------------------------------'
!PRINT *,'Starting to explore the identification region'
PRINT *,'True parameters:'
PRINT '("Theta: ",2(F10.6))',theta1,theta2
PRINT '("Beta1: ",3(F10.6))',beta1
PRINT '("Beta2: ",3(F10.6))',beta2
PRINT '("alpha: ",2(F10.6))',alpha_01,alpha_10
PRINT *,'  '
PRINT *,'Start time:'
CALL DATE_AND_TIME(DATE,TIME)
PRINT *,TIME
PRINT *,'----------------------------------------------------------'



!!!!!!!!!!!!!!!!!!! Differential Evolution stage !!!!!!!!!!!!!!!!!!!!!!!!!!
!initiate the population by randomly choosing Np points from the domain
CALL RANDOM_NUMBER(POP_loc) !this is from uniform [0,1]
DO i=1,Np
    POP_loc(i,:)=(DHIGH+DLOW)/TWO+POP_loc(i,:)*(DHIGH-DLOW)/4.0D0 !rescale
END DO

METHOD=2 !1=box method, 2=torus method

!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!! ABJ !!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!
PRINT *,"Start ABJ"
count=0
Center=ZERO
DO
    IF (count.GE.(8*N_iter)) EXIT  !(n_ABJmix.GT.2*Np) EXIT
    count=count+1
    DO i=1,Np   !<--go over the members of the population of points
        !create an alternative to POP_loc(i,:) to be put into y
        CALL make_y(i,y,method)
        IF (count.EQ.1 .AND. i.EQ.1) y=(/theta1, theta2, beta1, beta2/)  !<---injecting the true point
        !check y for all the possible values of the covariates X1 and X2 
        CALL is_mixABJ(y,ABJ)
        IF (ABJ) THEN
            POP_loc(i,1:par_dim)=y
            IF (count.LT.4*N_iter) THEN 
                n_ABJmix=n_ABJmix+1
                ABJmin=MIN(ABJmin,y)
                ABJmax=MAX(ABJmax,y)
                WRITE(Unit=6,FMT=40) 1,y
                Center=((n_ABJmix-1)*Center+POP_loc(i,:))/n_ABJmix
            ELSE
                currentdist=MAXVAL(ABS(POP_loc(i,:)-center))
                maxdist=MAX(maxdist,currentdist)
                IF (currentdist.LT.HALF*maxdist) POP_loc(i,:)=3.0D0*POP_loc(i,:)-TWO*Center
                IF (currentdist.GT.0.8D0*maxdist) THEN
                    !!CLS stage
                    lambda=ONE
                    DO k=1,N_CLS
                        ytemp=(ONE-lambda)*POP_loc(i,:)+lambda*(gamma*(DHIGH-DLOW)+DLOW)
                        CALL is_mixABJ(ytemp,ABJtemp)
                        IF (ABJtemp) THEN
                            n_ABJmix=n_ABJmix+1
                            ABJmin=MIN(ABJmin,y)
                            ABJmax=MAX(ABJmax,y)
                            WRITE(Unit=6,FMT=40) 1,ytemp
                        END IF
                        gamma=4.0D0*gamma*(ONE-gamma)
                        lambda=ONE-((REAL(k,kind=8)-ONE)/REAL(k,kind=8))**acc
                    END DO !k
                    !IF (ABJtemp) POP_loc(i,:)=ytemp
                END IF
            END IF
        END IF
    END DO !i
    IF (MOD(count,1000).EQ.0) PRINT *,count,n_ABJmix
END DO
PRINT *,"End ABJ"
!DO i=1,Np
!PRINT '(8(F10.6))',Pop_loc(i,:)
!END DO

!initiate the population by randomly choosing Np points from the domain
!CALL RANDOM_NUMBER(POP_loc) !this is from uniform [0,1]
!DO i=1,Np
!    POP_loc(i,:)=POP_loc(i,:)*(DHIGH-DLOW)+DLOW !rescale
!END DO

!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!! CT  !!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!
PRINT *,"Start CT"
count=0
lambda=ONE
DLOW=ABJmin
DHIGH=ABJmax
CENTER=ZERO
maxdist=ZERO
DO
    IF (count.GE.(2*N_iter)) EXIT  
    count=count+1
    DO i=1,Np   !<--go over the members of the population of points
        !create an alternative to POP_loc(i,:) to be put into y
        CALL make_y(i,y,METHOD)
        IF (count.EQ.1 .AND. i.EQ.1) y=(/theta1, theta2, beta1, beta2/) !<---injecting the true point
        CALL is_mixCT(y,CT)
        IF (CT) THEN
            POP_loc(i,1:par_dim)=y
            IF (count.LT.N_iter) THEN 
                n_CTmix=n_CTmix+1
                CTmin=MIN(CTmin,y)
                CTmax=MAX(CTmax,y)
                WRITE(Unit=6,FMT=40) 2,y
                Center=((n_CTmix-1)*Center+POP_loc(i,:))/n_CTmix
            ELSE
                currentdist=MAXVAL(ABS(POP_loc(i,:)-center))
                maxdist=MAX(maxdist,currentdist)
                IF (currentdist.LT.HALF*maxdist) POP_loc(i,:)=3.0D0*POP_loc(i,:)-TWO*Center
                IF (currentdist.GT.0.8D0*maxdist) THEN
                    !!CLS stage
                    lambda=ONE
                    DO k=1,N_CLS
                        ytemp=(ONE-lambda)*POP_loc(i,:)+lambda*(gamma*(DHIGH-DLOW)+DLOW)
                        CALL is_mixCT(ytemp,CTtemp)
                        IF (CTtemp) THEN
                            n_CTmix=n_CTmix+1
                            CTmin=MIN(CTmin,ytemp)
                            CTmax=MAX(CTmax,ytemp)
                            WRITE(Unit=6,FMT=40) 2,ytemp
                        END IF
                        gamma=4.0D0*gamma*(ONE-gamma)
                        lambda=ONE-((REAL(k,kind=8)-ONE)/REAL(k,kind=8))**acc
                    END DO !k
                    !IF (ABJtemp) POP_loc(i,:)=ytemp
                END IF
            END IF
        END IF
    END DO !i
    IF (MOD(count,1000).EQ.0) PRINT *,count,n_CTmix
    gamma=4.0D0*gamma*(ONE-gamma)
    lambda=ONE-((REAL(n_CTmix,kind=8)-ONE)/REAL(n_CTmix,kind=8))**acc
END DO
PRINT *,"End CT"

!initiate the population by randomly choosing Np points from the domain
!CALL RANDOM_NUMBER(POP_loc) !this is from uniform [0,1]
!DO i=1,Np
!    POP_loc(i,:)=POP_loc(i,:)*(DHIGH-DLOW)+DLOW !rescale
!END DO

!GOTO 999
!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!! BMM !!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!
PRINT *,"Start BMM"
count=0
lambda=ONE
DLOW=CTmin
DHIGH=CTmax
CENTER=ZERO
maxdist=ZERO
DO
    IF (count.GE.2*N_iter) EXIT  
    count=count+1
    DO i=1,Np   !<--go over the members of the population of points
        !create an alternative to POP_loc(i,:) to be put into y
        CALL make_y(i,y,METHOD)
        IF (count.EQ.1 .AND. i.EQ.1) y=(/theta1, theta2, beta1, beta2/) !<---injecting the true point
        !done creating the alternative to POP_loc(i,:) and it's in y
            CALL is_mixCT(y,CT)
            IF (CT) THEN
                CALL is_mixBMM(y,BMM)
            ELSE
                BMM=.FALSE.
            END IF
        IF (BMM) THEN
            POP_loc(i,1:par_dim)=y
            IF (count.LT.N_iter) THEN 
                n_BMMmix=n_BMMmix+1
                BMMmin=MIN(BMMmin,y)
                BMMmax=MAX(BMMmax,y)
                WRITE(Unit=6,FMT=40) 3,y
                Center=((n_BMMmix-1)*Center+POP_loc(i,:))/n_BMMmix
            ELSE
                currentdist=MAXVAL(ABS(POP_loc(i,:)-center))
                maxdist=MAX(maxdist,currentdist)
                IF (currentdist.LT.HALF*maxdist) POP_loc(i,:)=3.0D0*POP_loc(i,:)-TWO*Center
                IF (currentdist.GT.0.8D0*maxdist) THEN
                    !!CLS stage
                    lambda=ONE
                    DO k=1,N_CLS
                        ytemp=(ONE-lambda)*POP_loc(i,:)+lambda*(gamma*(DHIGH-DLOW)+DLOW)
                        CALL is_mixCT(ytemp,CTtemp)
                        IF (CTtemp) THEN
                            CALL is_mixBMM(ytemp,BMMtemp)
                        ELSE
                            BMMtemp=.FALSE.
                        END IF
                        IF (BMMtemp) THEN
                            n_BMMmix=n_BMMmix+1
                            BMMmin=MIN(BMMmin,ytemp)
                            CTmax=MAX(BMMmax,ytemp)
                            WRITE(Unit=6,FMT=40) 3,ytemp
                        END IF
                        gamma=4.0D0*gamma*(ONE-gamma)
                        lambda=ONE-((REAL(k,kind=8)-ONE)/REAL(k,kind=8))**acc
                    END DO !k
                    !IF (BMMtemp) POP_loc(i,:)=ytemp
                END IF
            END IF
        END IF
    END DO !i
IF (MOD(count,100).EQ.0) PRINT *,count,n_BMMmix
END DO
PRINT *,"End BMM"
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!! DE is done !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!999 CONTINUE

!!!!!!!!!!!!!!!!! OUTPUT !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!WRITE(Unit=6,FMT=55) theta1,theta2,alpha_01,alpha_10,n_ABJmix,n_CTmix,n_BMMmix,ABJmin,ABJmax,CTmin,CTmax,BMMmin,BMMmax
PRINT *,'points were found under BMM,CT and ABJ mixed estimator: '
PRINT *,n_BMMmix,n_CTmix,n_ABJmix


PRINT *,' '
PRINT *,'done simulations'
PRINT *,' '
!PRINT *,'Total points checked: '
!PRINT *,n
!WRITE(Unit=6,FMT=35) theta1,theta2,n_BMMmix,n_CTmix,n_ABJmix
PRINT *,'--------------------------------------'
PRINT *,'End time:'
CALL DATE_AND_TIME(DATE,TIME)
PRINT *,TIME
PRINT *,'--------------------------------------'
ENDFILE 6

END PROGRAM mixnash