PROGRAM trycorrnorm

! Two players, no covariates, Correlated Equilibrium
! This program uses the module corr_module.f90 and input files epsilon_norm.txt and directions.txt
! The program produces the output file corr.txt
!
! Arie Beresteanu, Ilya Molchanov and Francesca Molinari (Econometrica, 2011): "Sharp Identification Regions in Models with Convex Moment Predictions,"

USE corrfunctions
IMPLICIT NONE

!Variable definitions
REAL (dp) :: theta1,theta2
REAL (dp) :: p00,p01,p10,p11,pb,pc,pd,pe,th1,th2,th2_temp
REAL (dp) :: u1,u2,u3,u4,e1,e2,eps1,eps2,temp,E  !,f
REAL (dp), DIMENSION(4) :: u,P
REAL (dp), DIMENSION(4,1) :: Pvec
INTEGER(KIND=8)   :: n_BMMmix, n_CTmix, n_ABJmix
INTEGER(KIND=8)   :: c, ios,n,I
LOGICAL(KIND=4)   :: ABJ,CT,BMM
CHARACTER*10 :: TIME
CHARACTER*8  :: DATE


!!!!!!!!!!  Reading directions !!!!!!!!!!!!!!!!!!!!!!!!
OPEN(Unit=9,file='directions.txt',IOSTAT=ios)
IF (ios.ne.0) THEN
  PRINT *,'error reading in file'
  STOP
ENDIF

30 FORMAT (4(F15.10))
35 FORMAT (2(F15.10))

DO I=1,N_tiles
    READ(Unit=9,FMT=30) u1,u2,u3,u4
    Umat(I,:)=(/u1,u2,u3,u4/)
    !print *,u1,u2,u3,u4
END DO
CLOSE(Unit=9)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!! Reading epsilon vector !!!!!!!!!!!!!!!!!!!!!!
OPEN(Unit=9,file='epsilon_norm.txt',IOSTAT=ios)
IF (ios.ne.0) THEN
  PRINT *,'error reading in file'
  STOP
ENDIF

DO I=1,N_epsilon
    READ(Unit=9,FMT=35) e1,e2
    Eps(I,:)=(/e1,e2/)
    !print *,e1,e2
END DO
CLOSE(Unit=9)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! pick true parameters
theta1=-1.15
theta2=-1.4

!the followingvalues were computed in Matlab for the above selected theta1 and theta2

!PB=pb_small+0.146612245142232D0
PB=pb_small+0.0524355761366524D0
!PC=pc_small+0.0179380639828147D0
PC=pc_small+0.030277937932606D0
!PE=0.0998700687699396D0
PE=0.157186094250506D0
PD=ONE-PA-PB-PC-PE

! create a selection
P00=PA+0.10D0*PE
P10=PC+0.40D0*PE
P01=PB+0.40D0*PE
P11=ONE-P00-P10-P01
P=(/P00, P10, P01, P11/)
Pvec(:,1)=P
f=MATMUL(Umat,Pvec)

!PRINT *,P

PRINT *, ' '
PRINT *,'----------------------------------------------------------'
PRINT *, 'Starting to explore the identification region'
PRINT '("Number of tiles on the unit sphere: ",I8)',N_tiles
Print '("True parameter: ",F8.4,F8.4)', theta1,theta2
PRINT *,'Start time:'
CALL DATE_AND_TIME(DATE,TIME)
PRINT *,TIME
PRINT *,'----------------------------------------------------------'
PRINT *,' '


OPEN(Unit=6,file='corr.txt',IOSTAT=ios)
IF (ios.ne.0) THEN
  PRINT *,'error writting output file'
  STOP
ENDIF

40 FORMAT (1(I6),2(F15.10))


! provide a parameter candidates to check
n_BMMmix=0 !set the initial length of BMM_mix (i.e. how may parameters were found so far)
n_CTmix=0 
n_ABJmix=0
n=0
DO th1=-5.005,-0.005,0.01
    DO th2=-5.005,-0.005,0.01
        n=n+1
        CALL is_corr(th1,th2,P,ABJ,CT,BMM)
        !PRINT *, th1, th2
        IF (BMM.EQ..TRUE.) THEN
            n_BMMmix=n_BMMmix+1
            n_CTmix=n_CTmix+1
            n_ABJmix=n_ABJmix+1
            WRITE(Unit=6,FMT=40) 3,th1,th2
        ELSE
            IF (CT.EQ..TRUE.) THEN
                n_CTmix=n_CTmix+1
                n_ABJmix=n_ABJmix+1
                WRITE(Unit=6,FMT=40) 2,th1,th2
            ELSE
                IF (ABJ.EQ..TRUE.) THEN
                    n_ABJmix=n_ABJmix+1
                    WRITE(Unit=6,FMT=40) 1,th1,th2
                END IF
            END IF
        END IF
    END DO
    PRINT *, th1, n_BMMmix ,n_CTmix, n_ABJmix
END DO
      
CLOSE(Unit=6)
PRINT *,' '
PRINT *,'done simulations'
PRINT *,' '
PRINT *, 'Total points checked: ', n
PRINT *, n_BMMmix, 'points were found under BMM mixed estimator'
PRINT *, n_CTmix, 'points were found under CT mixed estimator'
PRINT *, n_ABJmix, 'points were found under ABJ mixed estimator'
PRINT *,'--------------------------------------'
PRINT *,'End time:'
CALL DATE_AND_TIME(DATE,TIME)
PRINT *,TIME
PRINT *,'--------------------------------------'
PRINT *,'done saving results under: idregions.txt'

END PROGRAM trycorrnorm              