MODULE corrfunctions

! Two players, no covariates, Correlated Equilibrium
! This program is the module used by corr.f90 
!
! Arie Beresteanu, Ilya Molchanov and Francesca Molinari (Econometrica, 2011): "Sharp Identification Regions in Models with Convex Moment Predictions,"


IMPLICIT NONE
SAVE

INTEGER, PARAMETER  :: dp = SELECTED_REAL_KIND(12, 32)
INTEGER(KIND=4), PARAMETER :: N_tiles=1160  !number of tiles on the unit sphere (to be chosen by the user)
INTEGER(KIND=4), PARAMETER :: N_epsilon=19956
REAL (dp), DIMENSION(N_epsilon,2) :: Eps
REAL (dp), DIMENSION(N_tiles,4) :: Umat  !this contains the directions that were created by a separate program genU.f90
REAL (dp), DIMENSION(N_tiles,1) :: f
REAL (dp), PARAMETER :: ONE=1.0D0, Zero=0.0D0
!REAL (dp), PARAMETER :: pa=0.307513363D0, pb_small=0.192486636D0, pc_small=0.192486636D0
REAL (dp), PARAMETER :: pa=0.250D0, pb_small=0.250D0, pc_small=0.250D0
REAL (dp), PARAMETER :: positive_ortant=ONE-pa-pb_small-pc_small

CONTAINS


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE is_corr(th1,th2,P,ABJ,CT,BMM)
! check whether (th1,th2) is in the identification region 
! P - vector of empirical frequencies


REAL (dp), INTENT(IN) :: th1,th2
REAL (dp), INTENT(IN), DIMENSION(4) :: P
LOGICAL(KIND=4), INTENT(OUT) :: ABJ,CT,BMM

REAL (dp), DIMENSION(N_epsilon,4) :: gamma1,gamma2,gamma3
REAL (dp), DIMENSION(4) :: u
REAL (dp) :: pb,pc,pd,pe,pb1,pc1,pd1,pe1
REAL (dp) :: u1,u2,u3,u4,temp,E 
REAL (dp) :: c ,eps1,eps2
INTEGER(KIND=8)   :: k, j, ne

!initial values for the region's probability mass

!first compute regions
pb1=ZERO
pc1=ZERO
pd1=ZERO
pe1=ZERO
ne=0
DO j=1,N_epsilon
   eps1=Eps(j,1)
   eps2=Eps(j,2)
   IF (eps1.GT.(-th1)) THEN
        IF ( eps2.GT.(-th2)) THEN
            pd1=pd1+ONE
        ELSE
            pb1=pb1+ONE
        END IF
   ELSE
        IF (eps2.GT.(-th2)) THEN
            pc1=pc1+ONE
        ELSE
            pe1=pe1+ONE
            ne=ne+1
            !new_eps(ne,:)=(/eps1,eps2/)
            gamma1(ne,:)=(/ONE, -(eps2/(th2+eps2)), -(eps1/(th1+eps1)), ZERO/) / (ONE-eps2/(th2+eps2)-eps1/(th1+eps1))
            gamma2(ne,:)=(/(ONE+eps2/th2)*(ONE+eps1/th1), -(eps2/th2)*(ONE+eps1/th1), -(ONE+eps2/th2)*(eps1/th1), (eps2*eps1)/(th1*th2)/)
            gamma3(ne,:)=(/ZERO, -(eps2/(th2+eps2)), -(eps1/(th1+eps1)), (eps1*eps2/((th2+eps2)*(th1+eps1)))/) / (eps1*eps2/((th2+eps2)*(th1+eps1))-eps2/(th2+eps2)-eps1/(th1+eps1))
        END IF
   END IF
END DO

pb=pb_small+pb1/REAL(N_epsilon,dp)*positive_ortant
pc=pc_small+pc1/REAL(N_epsilon,dp)*positive_ortant
pd=pd1/REAL(N_epsilon,dp)*positive_ortant
pe=pe1/REAL(N_epsilon,dp)*positive_ortant
!PRINT *,pb,pc,pd,pe,ne

! check in all N_tiles directions
ABJ=.TRUE.
CT =.TRUE.
BMM=.TRUE.
DO k=1,N_tiles
    u=Umat(k,:)
    temp=ZERO
    c=ZERO
    DO j=1,ne
        !eps1=new_eps(j,1)
        !eps2=new_eps(j,2)
        !gamma1=(/ONE, -(eps2/(th2+eps2)), -(eps1/(th1+eps1)), ZERO/) / (ONE-eps2/(th2+eps2)-eps1/(th1+eps1))
        !gamma2=(/(ONE+eps2/th2)*(ONE+eps1/th1), -(eps2/th2)*(ONE+eps1/th1), -(ONE+eps2/th2)*(eps1/th1), (eps2*eps1)/(th1*th2)/)
        !gamma3=(/ZERO, -(eps2/(th2+eps2)), -(eps1/(th1+eps1)), (eps1*eps2/((th2+eps2)*(th1+eps1)))/) / (eps1*eps2/((th2+eps2)*(th1+eps1))-eps2/(th2+eps2)-eps1/(th1+eps1))
        temp=temp+MAX(u(2),u(3),DOT_PRODUCT(u,gamma1(j,:)),DOT_PRODUCT(u,gamma2(j,:)),DOT_PRODUCT(u,gamma3(j,:)))
        c=c+ONE
    END DO
    E=u(1)*pa+u(2)*pc+u(3)*pb+u(4)*pd+(temp/c)*pe
    !f=DOT_PRODUCT(u,P)
    IF (E.LT.f(k,1)) THEN
        SELECT CASE (k)
            CASE (1:4)
                ABJ=.FALSE.
                CT=.FALSE.
                BMM=.FALSE.
                RETURN
            CASE (5:8)
                CT=.FALSE.
                BMM=.FALSE.
                RETURN
            CASE (9:)
                BMM=.FALSE.
                RETURN
        END SELECT
    END IF
END DO

END SUBROUTINE is_corr
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


END MODULE corrfunctions            