PROGRAM twotypes

! Four players (two types), no covariates, pure Nash Equilibrium
!
! This program calls the module called by twotypesfunctions.f90 and the input file theta.txt
! The program produces the output file idregions_opt.txt
!
! Arie Beresteanu, Ilya Molchanov and Francesca Molinari (Econometrica, 2011): "Sharp Identification Regions in Models with Convex Moment Predictions,"



USE twotypesfunctions
IMPLICIT NONE

REAL (dp) :: th11,th21,th22,t1,t2,t3
INTEGER :: I,ios,n_BMM
LOGICAL :: BMM
CHARACTER*10 :: TIME
CHARACTER*8  :: DATE
CHARACTER*15 :: filename

!30 FORMAT (2(F15.10))
40 FORMAT (3(F20.15))

!!!!!!!!!!! Reading theta triplets !!!!!!!!!!!!!!!!!!!!!
OPEN(Unit=9,file='theta.txt',IOSTAT=ios)
IF (ios.ne.0) THEN
  PRINT *,'error reading in file'
  STOP
ENDIF

DO i=1,N_theta
    READ(Unit=9,FMT=40) t1,t2,t3
    Theta(i,:)=(/t1,t2,t3/)
    !print *,i,t1,t2,t3
END DO
CLOSE(Unit=9)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!! Open output file !!!!!!!!!!!!!!!!!!!!!
OPEN(Unit=6,file='idregion_opt.txt',IOSTAT=ios)
IF (ios.ne.0) THEN
  PRINT *,'error writting output file'
  STOP
ENDIF
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

PRINT *,'----------------------------------------------------------'
PRINT *,'Starting to explore the identification region'
!PRINT *,'True parameter:'
!PRINT *,theta1,theta2
PRINT *,'  '
PRINT *,'Start time:'
CALL DATE_AND_TIME(DATE,TIME)
PRINT *,TIME
PRINT *,'----------------------------------------------------------'

Ytheta(1,1:6)=(/1, 1,  4,  7,  7,  7/)
Ytheta(2,1:6)=(/1, 1,  4,  7,  7,  7/)
Ytheta(3,1:6)=(/2, 2, 24,  7,  7,  7/)
Ytheta(4,1:6)=(/3, 3, 34, 37,  7,  7/)
Ytheta(5,1:6)=(/3, 3,  3,357,  7,  7/)
Ytheta(6,1:6)=(/3, 3,  3, 37, 67,  7/)
Ytheta(7,1:6)=(/3, 3,  3,  3, 68,  8/)
Ytheta(8,1:6)=(/3, 3,  3,  3,  6,  9/)


P=(/0.3021, 0.0231, 0.2779, 0.0335, 0.0019, 0.0158, 0.2601, 0.0104, 0.0752/)
!     P00     P01    P02      P10     P11     P12     P20     P21     P22

n_BMM=0
DO i=1,N_theta  !go over all the theta triplets
!DO i=16421,16421
    th11=Theta(i,1)
    th21=Theta(i,2)
    th22=Theta(i,3)
    BMM=isminneg2(th11,th21,th22)
    IF (BMM.EQ..TRUE.) THEN
         n_BMM=n_BMM+1
         WRITE(Unit=6,FMT=40) th11,th21,th22
!PRINT *, th11,th21,th22
    END IF
END DO
PRINT *,"found: ",n_BMM," points"
PRINT *,'----------------------------------------------------------'
PRINT *,'Finish time:'
CALL DATE_AND_TIME(DATE,TIME)
PRINT *,TIME
PRINT *,'----------------------------------------------------------'

END PROGRAM twotypes