MODULE twotypesfunctions

! Four players (two types), no covariates, pure Nash Equilibrium
!
! This module is called by the main program twotypes.f90
! The program produces the output file idregions_opt.txt
!
! Arie Beresteanu, Ilya Molchanov and Francesca Molinari (Econometrica, 2011): "Sharp Identification Regions in Models with Convex Moment Predictions,"


IMPLICIT NONE
SAVE

INTEGER, PARAMETER :: N_Theta=26421
REAL, DIMENSION(N_Theta,3) :: Theta
INTEGER, PARAMETER  :: dp = SELECTED_REAL_KIND(16, 24)
REAL (dp), DIMENSION(8,6) :: Ytheta
REAL (dp), DIMENSION(9) :: P
REAL (dp), PARAMETER :: ONE=1.0D+0, HALF=0.50D+0, ZERO=0.0D+0, TWO=2.0D+0, THREE=3.0D+0, FOUR=4.0D+0
REAL (dp), PARAMETER :: t=0.00000001D0

CONTAINS


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
REAL (dp) FUNCTION vf(th11,th21,th22,u)

REAL (dp), INTENT (IN), DIMENSION(9) :: u
REAL (dp), INTENT(IN) :: th11,th21,th22
REAL (dp), DIMENSION(9) :: threshold1
REAL (dp), DIMENSION(7) :: threshold2
REAL (dp) :: E,Etemp, minu
INTEGER :: i,j1,j2,temp,eq

!thresholds for epsilon1:
threshold1(1)=ONE
threshold1(2)=HALF
threshold1(3)=alnorm(th22,.FALSE.)
threshold1(4)=alnorm(TWO*th22,.FALSE.)
threshold1(5)=alnorm(th21+th22,.FALSE.)
threshold1(6)=alnorm(th21+TWO*th22,.FALSE.)
threshold1(7)=alnorm(TWO*th21+th22,.FALSE.)
threshold1(8)=alnorm(TWO*th21+TWO*th22,.FALSE.)  
threshold1(9)=ZERO    

!thresholds for epsilon2:
threshold2(1)=ONE
threshold2(2)=HALF
threshold2(3)=alnorm(th11,.FALSE.)
threshold2(4)=alnorm(TWO*th11,.FALSE.)
threshold2(5)=alnorm(THREE*th11,.FALSE.)
threshold2(6)=alnorm(FOUR*th11,.FALSE.)
threshold2(7)=ZERO

!PRINT *,threshold1
!PRINT *,threshold2
minu=MINVAL(u)
E=ZERO
DO j1=1,8
    DO j2=1,6
        Etemp=minu
        temp=Ytheta(j1,j2)
        DO 
            eq=MOD(temp,10)
            Etemp=MAX(Etemp,u(eq))
            temp=INT(temp/10)
            IF (temp.EQ.0) EXIT
        END DO
        E=E+Etemp*(threshold1(j1)-threshold1(j1+1))*(threshold2(j2)-threshold2(j2+1))
    END DO
END DO
!PRINT *,E
!Print *,DOT_PRODUCT(P,u)
!Print *,"========="
vf=E-DOT_PRODUCT(P,u)
END FUNCTION vf
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
LOGICAL FUNCTION isminneg2(th11,th21,th22)

REAL (dp), INTENT(IN) :: th11,th21,th22


REAL (dp), DIMENSION(10,10) :: U
REAL (dp), DIMENSION(9) :: M,R,E,C,S
REAL (dp) :: zM,zR,zE,zC,zS
INTEGER :: j,cc,ii,jj

cc=0
U=ZERO
DO j=1,9
    U(j,j)=ONE
    U(j,10)=vf(th11,th21,th22,U(j,1:9))
END DO
! order the points by the fifth column
CALL inssor(U)
!PRINT '(10(F12.10))',((U(ii,jj),jj=1,10),ii=1,10)
DO
    cc=cc+1
    !PRINT *,cc,U(10,10),U(1,10)
    IF ((U(1,10)<-t) .OR. ((U(10,10)-U(1,10))<t)) EXIT  !we either encountered a negative value or reached convergence
    ! defining new points
    M=(U(1,1:9)+U(2,1:9)+U(3,1:9)+U(4,1:9)+U(5,1:9)+U(6,1:9)+U(7,1:9)+U(8,1:9)+U(9,1:9))/9.0D+0 !average of all but the worse point
    zM=vf(th11,th21,th22,M)
    R=TWO*M-U(10,1:9)
    zR=vf(th11,th21,th22,R)
    IF (zR<U(9,10))  THEN  !case 1
        IF (U(1,10)<zR) THEN
            U(10,1:9)=R
            U(10,10)= zR
        ELSE
            E=TWO*R-M
            zE=vf(th11,th21,th22,E)
            IF (zE<U(1,10)) THEN
                U(10,1:9)=E
                U(10,10)=zE
            ELSE
                U(10,1:9)=R
                U(10,10)=zR
            END IF
        END IF
    ELSE   !case 2
        IF (zR<U(10,10)) THEN
            U(10,1:9)=R
            U(10,10)=zR
        END IF
        C=(U(10,1:9)+M)/TWO
        zC=vf(th11,th21,th22,C)
        IF (zC<U(10,10)) THEN
            U(10,1:9)=C
            U(10,10)=zC
        ELSE
            S=(U(1,1:9)+U(10,1:9))/TWO
            zS=vf(th11,th21,th22,S)
            U(10,1:9)=S
            U(10,10)=zS
            U(9,1:9)=M
            U(9,10)=zM
        END IF
    END IF
    ! order the points to create BGW
    CALL inssor(U)
END DO
!PRINT *,cc,U(10,10),U(1,10)

isminneg2=(U(1,10)>=-t) !is the best value non-negative?
!PRINT *,U(1,10),-t,isminneg2

END FUNCTION isminneg2
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE inssor(U)
!  Sorts U into increasing order (Insertion sort) by the tenth column
      REAL (dp), DIMENSION (10,10), INTENT (InOut) :: U
      REAL (dp), DIMENSION (10) :: XWRK, XMIN
      INTEGER :: ICRS, IDCR
!
! We first bring the minimum to the first location in the array.
! That way, we will have a "guard", and when looking for the
! right place to insert a value, no loop test is necessary.
!
      IF (U(1,10) < U(10,10)) THEN
          XMIN = U(1,1:10)
      ELSE
          XMIN = U(10,1:10)
          U(10,1:10) = U(1,1:10)
      END IF
      DO IDCR = 9, 2, -1
         XWRK = U(IDCR,1:10)
         IF (XWRK(10) < XMIN(10)) THEN
            U(IDCR,1:10) = XMIN
            XMIN = XWRK
         END IF
      END DO
      U(1,1:10) = XMIN
!
! The first value is now the minimum
! Loop over the array, and when a value is smaller than
! the previous one, loop down to insert it at its right place.
!
      DO ICRS = 3, 10
         XWRK = U(ICRS,1:10)
         IDCR = ICRS - 1
         IF (XWRK(10) < U(IDCR,10)) THEN
            U(ICRS,1:10) = U(IDCR,1:10)
            IDCR = IDCR - 1
            DO
               IF (XWRK(10) >= U(IDCR,10)) EXIT
               U(IDCR+1,1:10) = U(IDCR,1:10)
               IDCR = IDCR - 1
            END DO
            U(IDCR+1,1:10) = XWRK
         END IF
      END DO
!
      RETURN
!
END SUBROUTINE inssor
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!



!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
FUNCTION alnorm ( x, upper )
!*****************************************************************************
!
!! ALNORM computes the cumulative density of the standard normal distribution.
!
!  Modified:
!
!    13 January 2008
!
!  Author:
!
!    David Hill
!    FORTRAN90 version by John Burkardt
!
!  Reference:
!
!    David Hill,
!    Algorithm AS 66:
!    The Normal Integral,
!    Applied Statistics,
!    Volume 22, Number 3, 1973, pages 424-427.
!
!  Parameters:
!
!    Input, real ( kind = 8 ) X, is one endpoint of the semi-infinite interval
!    over which the integration takes place.
!
!    Input, logical UPPER, determines whether the upper or lower
!    interval is to be integrated:
!    .TRUE.  => integrate from X to + Infinity;
!    .FALSE. => integrate from - Infinity to X.
!
!    Output, real ( kind = 8 ) ALNORM, the integral of the standard normal
!    distribution over the desired interval.
!
  IMPLICIT NONE

  REAL ( dp ), PARAMETER :: a1 = 5.75885480458D+00
  REAL ( dp ), PARAMETER :: a2 = 2.62433121679D+00
  REAL ( dp ), PARAMETER :: a3 = 5.92885724438D+00
  REAL ( dp ) alnorm
  REAL ( dp ), PARAMETER :: b1 = -29.8213557807D+00
  REAL ( dp ), PARAMETER :: b2 = 48.6959930692D+00
  REAL ( dp ), PARAMETER :: c1 = -0.000000038052D+00
  REAL ( dp ), PARAMETER :: c2 = 0.000398064794D+00
  REAL ( dp ), PARAMETER :: c3 = -0.151679116635D+00
  REAL ( dp ), PARAMETER :: c4 = 4.8385912808D+00
  REAL ( dp ), PARAMETER :: c5 = 0.742380924027D+00
  REAL ( dp ), PARAMETER :: c6 = 3.99019417011D+00
  REAL ( dp ), PARAMETER :: con = 1.28D+00
  REAL ( dp ), PARAMETER :: d1 = 1.00000615302D+00
  REAL ( dp ), PARAMETER :: d2 = 1.98615381364D+00
  REAL ( dp ), PARAMETER :: d3 = 5.29330324926D+00
  REAL ( dp ), PARAMETER :: d4 = -15.1508972451D+00
  REAL ( dp ), PARAMETER :: d5 = 30.789933034D+00
  REAL ( dp ), PARAMETER :: ltone = 7.0D+00
  REAL ( dp ), PARAMETER :: p = 0.398942280444D+00
  REAL ( dp ), PARAMETER :: q = 0.39990348504D+00
  REAL ( dp ), PARAMETER :: r = 0.398942280385D+00
  LOGICAL :: up
  LOGICAL :: upper
  REAL ( dp ), PARAMETER :: utzero = 18.66D+00
  REAL ( dp ) x
  REAL ( dp ) y
  REAL ( dp ) z

  up = upper
  z = x

  IF ( z < 0.0D+00 ) THEN
    up = .not. up
    z = - z
  END IF

  IF ( ltone < z .and. ( ( .not. up ) .or. utzero < z ) ) THEN

    IF ( up ) THEN
      alnorm = 0.0D+00
    ELSE
      alnorm = 1.0D+00
    END IF

    RETURN

  END IF

  y = 0.5D+00 * z * z

  IF ( z <= con ) THEN

    alnorm = 0.5D+00 - z * ( p - q * y &
      / ( y + a1 + b1 &
      / ( y + a2 + b2 & 
      / ( y + a3 ))))

  ELSE

    alnorm = r * exp ( - y ) &
      / ( z + c1 + d1 &
      / ( z + c2 + d2 &
      / ( z + c3 + d3 &
      / ( z + c4 + d4 &
      / ( z + c5 + d5 &
      / ( z + c6 ))))))

  END IF

  IF ( .not. up ) THEN
    alnorm = 1.0D+00 - alnorm
  END IF

  RETURN
END FUNCTION alnorm
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!



END MODULE twotypesfunctions