MODULE func_module

IMPLICIT NONE
SAVE

INTEGER, PARAMETER  :: dp = SELECTED_REAL_KIND(12, 32)
REAL (dp) , PARAMETER :: pi = 3.141592653589793

CONTAINS

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
REAL (dp) FUNCTION normrnd()
!
! normrnd samples the standard normal probability distribution.
!  Method:
!    The Box-Muller method is used, which is efficient, but
!    generates two values at a time.
!  (modified version of a program by John Burkardt)

IMPLICIT NONE

REAL (dp) :: r1, r2
INTEGER, SAVE :: used = 0
REAL (dp) :: x
REAL (dp), SAVE :: y = 0.0D+00
!
!  If we've used an even number of values so far, generate two more,
!  return one and save one.
!
IF (mod(used,2).EQ.0) THEN
    CALL RANDOM_NUMBER(r1)
    CALL RANDOM_NUMBER(r2)
    x = sqrt(-2.0D+00 * log ( r1 ) ) * cos ( 2.0D+00 * pi * r2 )
    y = sqrt (-2.0D+00 * log ( r1 ) ) * sin ( 2.0D+00 * pi * r2 )
!
!  Otherwise, return the second, saved, value.
!
ELSE
    x = y
END IF
used = used + 1
normrnd = x
END FUNCTION normrnd

END MODULE func_module


PROGRAM genDELTAnorm

USE func_module
IMPLICIT NONE

INTEGER, PARAMETER :: n_eps=80000 ! the number of epsilons to choose
REAL (dp) :: pa,pb_small,pc_small,eps(1,2)
!REAL (dp) :: cholV(2,2)
INTEGER :: k,ios,n


OPEN(Unit=6,file='epsilon_norm.txt',IOSTAT=ios)
IF (ios.ne.0) THEN
  PRINT *,'error writting output file'
  STOP
ENDIF

40 FORMAT (2(F15.10))
pa=0.0
pb_small=0.0
pc_small=0.0
!cholV(1,1)=1.0
!cholV(1,2)=1.0/sqrt(2.0)
!cholV(2,1)=0.0
!cholV(2,2)=sqrt(3.5)
k=0

DO n=1,n_eps
    eps(1,1)=normrnd()
    eps(1,2)=normrnd()
!    eps=MATMUL(eps,cholV)
    IF (eps(1,1) .LT. 0.0 .AND. eps(1,2) .LT. 0.0) THEN
        pa=pa+1.0
    END IF
    IF (eps(1,1) .GT. 0.0 .AND. eps(1,2) .LT. 0.0) THEN
        pb_small=pb_small+1.0
    END IF
    IF (eps(1,1) .LT. 0.0 .AND. eps(1,2) .GT. 0.0) THEN
        pc_small=pc_small+1.0
    END IF
    IF (eps(1,1) .GT. 0.0 .AND. eps(1,2) .GT. 0.0) THEN
            WRITE(Unit=6,FMT=40) eps
            k=k+1 !just counts the number of epsilons
    END IF
END DO

CLOSE(Unit=6)
PRINT *,' '
PRINT *,'done generating delta vector'
PRINT *,' '
PRINT *, 'Total points generated: ', k
PRINT *,' '
PRINT *,'pa=',pa
PRINT *,'pb_small=',pb_small
PRINT *,'pc_small=',pc_small

END PROGRAM genDELTAnorm
